/*
 * lpdate.c
 * lessonplan
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>

static void 
lp_date_update_dmy (const GDate *const_d)
{
  GDate *d = (GDate *) const_d;
  GDateYear y;
  GDateMonth m;
  GDateDay day;
  
  guint32 A, B, C, D, E, M;
  
  g_return_if_fail (d != NULL);
  g_return_if_fail (d->julian);
  g_return_if_fail (!d->dmy);
  g_return_if_fail (g_date_valid_julian (d->julian_days));
  
  /* Formula taken from the Calendar FAQ; the formula was for the
   *  Julian Period which starts on 1 January 4713 BC, so we add
   *  1,721,425 to the number of days before doing the formula.
   *
   * I'm sure this can be simplified for our 1 January 1 AD period
   * start, but I can't figure out how to unpack the formula.  
   */
  
  A = d->julian_days + 1721425 + 32045;
  B = ( 4 *(A + 36524) )/ 146097 - 1;
  C = A - (146097 * B)/4;
  D = ( 4 * (C + 365) ) / 1461 - 1;
  E = C - ((1461*D) / 4);
  M = (5 * (E - 1) + 2)/153;
  
  m = M + 3 - (12*(M/10));
  day = E - (153*M + 2)/5;
  y = 100 * B + D - 4800 + (M/10);
  
#ifdef G_ENABLE_DEBUG
  if (!g_date_valid_dmy (day, m, y)) 
    {
      g_warning ("\nOOPS julian: %u  computed dmy: %u %u %u\n", 
		 d->julian_days, day, m, y);
    }
#endif
  
  d->month = m;
  d->day   = day;
  d->year  = y;
  
  d->dmy = TRUE;
}

guint        
lp_date_get_week_of_year (const GDate *d, int weekday)
{
// Monday = 1 thru Sunday = 7,  G_DATE_BAD_WEEKDAY = 0
  GDateWeekday wd;
  guint day;
  GDate first;
  
  g_return_val_if_fail (d != NULL, 0);
  g_return_val_if_fail (g_date_valid (d), 0);
  
  if (!d->dmy) 
    {
      lp_date_update_dmy (d);
    }
  g_return_val_if_fail (d->dmy, 0);  
  
  g_date_clear (&first, 1);
  
  g_date_set_dmy (&first, 1, 1, d->year);
  
  wd = g_date_get_weekday (&first) - 1; /* make Monday day 0 */
   if (wd == weekday) wd = 0;  
  
  day = g_date_get_day_of_year (d) - 1;
  
  return ((day + wd)/7U + (wd == 0 ? 1 : 0));
}

void 
lp_date_get_first_of_week(GDate *d, int first_day_of_week)
{
	/** Returns the first day of week
		Can be any day of the week.
		Monday =1 thru Sunday =7 **/
	GDateWeekday day_of_week = g_date_get_weekday (d);
	int day_difference;// to get difference between current day and first day of week
	if (first_day_of_week <= day_of_week)
	{
        day_difference = day_of_week - first_day_of_week;	
	}
	else
	{
       day_difference = 7-(first_day_of_week - day_of_week) ;  
	}
	g_date_subtract_days (d, day_difference);
}

gchar *
lp_date_get_date_string(GDate * mydate)
{
//This should format the date according to the user's system settings.
//TODO: Use this to format date strings.  Also, remember to change the way dates are represented in database.
      gchar mydate_string[101];
      g_date_strftime (mydate_string, 100, "%Y/%m/%d\0", mydate);
      return g_strdup(mydate_string);
}

gchar *
lp_date_get_sqlite_date_string(GDate * mydate)
{
//This should format the date according to the user's system settings.
//TODO: Use this to format date strings.  Also, remember to change the way dates are represented in database.
      gchar mydate_string[101];
      g_date_strftime (mydate_string, 100, "%Y-%m-%d\0", mydate);
      return g_strdup(mydate_string);
}

GDate *
lp_date_get_date_from_sqlite_format(gchar * date_string)
{
//This takes the sqlite formated DATE or DATETIME and converts it to a date.
	gchar** mydatetime = g_strsplit(date_string, " ", 2);  //in case the date_string is a DATETIME value
	GDate *mydate = g_date_new();
	g_date_set_parse(mydate, mydatetime[0]);
	g_strfreev(mydatetime);
	if(g_date_valid(mydate))
	{
		return mydate;
	}
	else
	{
		printf("***Error:(lp_date_get_date_string_from_sqlite_format) date not valid\n");
		return NULL;
	}
}

gchar *
lp_date_get_date_string_from_sqlite_format(gchar * date_string)
{
//This takes the sqlite formated DATE or DATETIME and converts it to the user's system settings.
	GDate *mydate = lp_date_get_date_from_sqlite_format(date_string);
	if(g_date_valid(mydate))
	{
		gchar *new_date_string = lp_date_get_date_string(mydate);
		g_date_free(mydate);
		return new_date_string;
	}
	else
	{
		printf("***Error:(lp_date_get_date_string_from_sqlite_format) date not valid\n");
		return NULL;
	}
}
